/*
 * Created on Mar 9, 2004
 * 
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
import javax.swing.JTextArea;
/**
 * @author kim
 * 
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class RecMazeSolver extends Thread implements MazeSolverInterface {
	protected static final int SLEEP_TIME = 1000; // delay for animation

	protected MazeGrid grid; 		// grid holding the maze
	protected Position start; 		// starting position in maze
	protected Position finish; 	// ending position in maze
	protected GraphicsMaze gm; 	// Applet program is running in
	protected int sleepTime; 		// How long to sleep between steps
	protected JTextArea stackDisplay; // Area displaying path so far
	
	protected Position current;	// where are in grid in search

	protected boolean success;		// Is there path to finish yet?

	// Initialize variables so ready to run solver as a separate thread.
	public RecMazeSolver(GraphicsMaze gm, MazeGrid grid, Position start,
							Position finish, int speed, JTextArea stackDisplay) {
		this.grid = grid;
		this.start = start;
		this.finish = finish;
		this.gm = gm;
		gm.showStatus("Starting to solve");
		this.setSpeed(speed);
		this.stackDisplay = stackDisplay;
	}

	/**
	 * Pre: Maze loaded into data and displayed on grid. 
	 * Post: Tried to solve the maze. Success is true iff succeeded in solving.
	 */
	public void run(){
		success = false;
		current = start;
		runMaze();
		if (success) {					// succeeded in solving maze
			// set color of finish cell back to finishColor
			grid.setFinish(finish);
			gm.showStatus("success!");
		} else { 						// No solution to maze
			gm.showStatus("failure");
		}
		System.out.println(gm.toString());		
	}
	
	/**
	 * Pre: Maze loaded into data and displayed on grid. Post: Tried to solve
	 * the maze. Success is true iff successed in solving.
	 */
	public void runMaze() {
		Position origCurrent = current; // Initialize current to start
		success = current.equals(finish); // No solution yet

		grid.visit(current); // visit the current position and save on stack
		stackDisplay.append("\n" + current.toString());
		try {
			while (current != null
					&& !success) { // Search until success or no empty cells
				sleep(sleepTime); // Pause to give better animation

				current = nextUnvisited(current); // Find new cell to visit
				if (current != null) { // If found cell to visit, move to it
					grid.visit(current);
					gm.showStatus( "Now at (" + current.getRow() + ","
							+ current.getCol() + ").");
					
					runMaze();
					current = origCurrent;		// reset current for next try
				} else {
					removeLastOnStackDisplay();
					grid.fail(origCurrent);
				}
			}
		} catch (InterruptedException exc) {
		}
	}

	/**
	 * post: Return next adjacent open and unvisited cell if there is one. Order
	 * attempted is north, east, south, west
	 */
	protected Position nextUnvisited(Position current) {
		Position nextTry = current.getNorth(); // Try north
		if (!grid.canVisit(nextTry)) { // If not a good cell to visit ...
			nextTry = current.getEast(); // Try east
			if (!grid.canVisit(nextTry)) { // If not a good cell to visit ...
				nextTry = current.getSouth(); // Try south
				if (!grid.canVisit(nextTry)) {
					// If not a good cell to visit ...
					nextTry = current.getWest(); // Try west
					if (!grid.canVisit(nextTry)) {
						// If not a good cell to visit ...
						nextTry = null; // No unvisited neighbors available
					}
				}
			}
		}
		return nextTry;
	}

	// post: Printed path from start to end and clear path
	protected void removeLastOnStackDisplay() {
		String text = stackDisplay.getText();
		int newLast = text.lastIndexOf('\n');
		stackDisplay.setText(text.substring(0, newLast));
	}

	// Adjust sleep time to reflect desired speed
	// Post:  Solver is set to move faster or slower depending on speed
	public void setSpeed(int speed) {
		sleepTime = SLEEP_TIME / speed;
	}
}
